#!/usr/bin/python
# 
# Demo code for AirRecorder output over ZMQ using 
# the ZMQ push/pull pattern.
#
# jgoff at arubanetworks.com
#
# ARUBA NETWORKS PROVIDES THE SOFTWARE "AS IS" AND WITHOUT WARRANTY OF ANY KIND, 
#    AND ARUBA NETWORKS HEREBY DISCLAIMS ALL EXPRESS OR IMPLIED WARRANTIES, INCLUDING 
#    WITHOUT LIMITATION WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
#    PURPOSE, PERFORMANCE, ACCURACY, RELIABILITY, AND NON-INFRINGEMENT. 
#    THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS AGREEMENT.
#
#	UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, INCLUDING, WITHOUT LIMITATION, 
#    TORT, CONTRACT, STRICT LIABILITY, OR OTHERWISE, SHALL ARUBA NETWORKS OR ITS 
#    LICENSORS, SUPPLIERS OR RESELLERS BE LIABLE TO YOU OR ANY OTHER PERSON FOR 
#    ANY INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER 
#    INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOST PROFITS, LOSS OF GOODWILL, 
#    WORK STOPPAGE, ACCURACY OF RESULTS, COMPUTER FAILURE OR MALFUNCTION, DAMAGES 
#    RESULTING FROM YOUR USE OF THE SOFTWARE.
  


import json
import re
try:
  import zmq
except ImportError:
  print "\n\nZMQ is missing, try \"pip install pyzmq\" or \"easy_install pyzmq\"\n\n"
  raise


if __name__ == '__main__':
  context = zmq.Context()
  client = context.socket(zmq.PULL)
  client.bind('tcp://127.0.0.1:5556')
  client.setsockopt(zmq.LINGER, 0)
  
  print "Start AirRecorder now using --log-store zmq option"
  while True:
    try:
      # On windows we need noblock else ctrl-c wont exit correctly
      # for other OS, blocking could be used if desired.
      msg = client.recv(zmq.NOBLOCK)
      
      if msg is None:
        continue

      # the message is formatted as noted in AirRecorder help
      # see "java -jar AirRecorder-1.3.11.jar --help"
      # for more detail
      
      
      # AR.RESULT\r\n
      #   {
	    #      "Source": <controller hostname/ip>
      #          "Status": <integer>,
      #          "LocalBeginTime": <local time in ms>,
      #          ...
      #          "Stdout": "the CLI output is here"
      #          ...
      #   }
      #
      
      # chop off the header and jsonify the rest
      (topic, jsonstring) = msg.split('\n', 1)
      data = json.loads(jsonstring)
      print "Received message topic [%s] from [%s]" % (topic, data["Source"])
      print json.dumps(data, sort_keys=False, indent=4) 
      
      # CLI output, if any, is in data["Stdout"]      
      if data["Stdout"]:
        # do something with the result, it is a string that can be
        # either split by "\n" and processed line by line, or it can be processed
        # as a string
        
        # example of show ap debug client table
        # first make sure it's the CLI command we want
        if re.match('show ap debug client-table', data["Command"]):
        
          # this kind of output should be processed line by line
          cliLines = data["Stdout"].split("\n")
          for line in cliLines:
            line = line.strip()   # get rid of \r

            # we can split this, using re.split which works for almost all fields 
            # except the timestamps which we are not interested in here
            values = re.split('\s+', line)
            
            # check that the line starts with a mac address (i.e. not part of the table header etc)
            if re.match('[0-9a-f]{2}:', line):
              # mac is in values[0], essid in values[1] etc, for this example 
              # we just want SNR which is in values[14] and values[15]
              print "Client [%s] on [%s %s] has SNR %s (%s)" % \
                (values[0],values[1],values[2],values[14],values[15])

        
    # for windows support  
    except zmq.ZMQError, e:
      if e.errno == zmq.EAGAIN:
        pass    # expected for NOBLOCK
      else:
        raise   # some other error or ctrl-c
